@@ -505,6 +505,7 @@ fn semantic_token_type_and_modifiers(
             HlMod::Consuming => semantic_tokens::CONSUMING,
             HlMod::Async => semantic_tokens::ASYNC,
             HlMod::Library => semantic_tokens::LIBRARY,
+            HlMod::Public => semantic_tokens::PUBLIC,
             HlMod::Unsafe => semantic_tokens::UNSAFE,
             HlMod::Callable => semantic_tokens::CALLABLE,
             HlMod::Static => lsp_types::SemanticTokenModifier::STATIC,
@@ -1187,6 +1188,7 @@ mod tests {
                         prefix_kind: PrefixKind::Plain,
                         enforce_granularity: true,
                         group: true,
+                        skip_glob_imports: true,
                     },
                 },
                 ide_db::base_db::FilePosition { file_id, offset },
